@@ -1,8 +1,6 @@
 # ================================ set_prior ===============================
 
-# Need to specify a model-specific list of built-in priors
-
-#' Construction of prior distributions
+#' Construction of prior distributions for extreme value model parameters
 #'
 #' Constructs a prior distribution for use as the argument \code{prior} in
 #' \code{\link{rpost}}.  The user can either specify their own prior function
@@ -25,11 +23,14 @@
 #'   then \code{model} gives the extreme value model to be used.  Using
 #'   either \code{model = "gev"}, \code{model = "pp"} or
 #'   \code{model = "os"} will result in the same (GEV) parameterisation.
-#'   \code{model} has no effect if \code{prior} is a function.
+#'   If \code{prior} is a function then the value of \code{model} is stored
+#'   so that in the subsequent call to \code{rpost}, consistency of the
+#'   prior and extreme value model parameterisations can be checked.
 #' @param ... Further arguments to be passed to the user-supplied or
 #'   in-built prior function.  For details of the latter see \strong{Details}.
 #' @details Of the in-built named priors available in revdbayes only
-#'   those specified using \code{prior = "norm"} are proper.  Other proper
+#'   those specified using \code{prior = "norm"} or
+#'   \code{prior = "loglognorm"} are proper.  Other proper
 #'   priors are available from the package evdbayes, via the
 #'   functions \code{\link[evdbayes]{prior.prob}},
 #'   \code{\link[evdbayes]{prior.quant}},
@@ -38,15 +39,15 @@
 #'   using \code{model = "gev"} and \code{prior = "norm"} in
 #'   \code{set_prior}.
 #'
-#'   The other in-built prior are improper, that is, the integral of the
+#'   The other in-built priors are improper, that is, the integral of the
 #'   prior function over its support is not finite.  Such priors do not
 #'   necessarily result in a proper posterior distribution. Northrop and
 #'   Attalides (2016) consider the issue of posterior propriety in Bayesian
 #'   extreme value analyses.  In most of improper priors below the prior for
 #'   the scale parameter \eqn{\sigma} is taken to be \eqn{1/\sigma},
 #'   i.e. a flat prior for \eqn{log \sigma}.  Here we denote the scale
-#'   parameter of the Generalised Pareto (GP) distribution by \eqn{\sigma},
-#'   whereas we use \eqn{\sigma_u} in the revdbayes vignette.
+#'   parameter of the GP distribution by \eqn{\sigma}, whereas we use
+#'   \eqn{\sigma_u} in the revdbayes vignette.
 #'
 #'   For all in-bulit priors the arguments \code{min_xi} and \code{max_xi} may
 #'   be supplied by the user.  The prior density is set to zero for any value
@@ -54,7 +55,8 @@
 #'   (\code{min_xi}, \code{max_xi}).  This will override the default values
 #'   of \code{min_xi} and \code{max_xi} in the named priors detailed above.
 #'
-#'   The names of the priors available and details of hyperparameters are:
+#'   \strong{Extreme value priors.} The names of the extreme value priors
+#'   set using \code{prior} and details of hyperparameters are:
 #' \itemize{
 #'   \item {\code{"norm"}.
 #'
@@ -79,14 +81,16 @@
 #'     of) the maximal data information (MDI) prior, that is,
 #'     \deqn{\pi(\sigma, \xi) = (1/ \sigma) exp[- a (\xi + 1)], for
 #'     \sigma > 0, \xi >= -1, a > 0.}
-#'     The MDI prior has \eqn{a = 1}.
+#'     The value of \eqn{a} is set using the argument \code{a}.  The default
+#'     value is \eqn{a = 1}, which gives the MDI prior.
 #'
 #'     For \code{model = "gev"}: (an extended version
 #'     of) the maximal data information (MDI) prior, that is,
 #'     \deqn{\pi(\mu, \sigma, \xi) = (1/ \sigma) exp[- a (\xi + 1)], for
 #'     \sigma > 0, \xi >= -1, a > 0.}
-#'     The MDI prior has \eqn{a = \gamma}, where \eqn{\gamma = 0.57721}
-#'     is Euler's constant
+#'     The value of \eqn{a} is set using the argument \code{a}.  The default
+#'     value is \eqn{a = \gamma}, where \eqn{\gamma = 0.57721} is Euler's
+#'     constant, which gives the MDI prior.
 #'
 #'     For each of these cases \eqn{\xi} must be is bounded below
 #'     \emph{a priori} for the posterior to be proper
@@ -104,14 +108,14 @@
 #'     \eqn{\xi} (and for \eqn{log \sigma}):
 #'     \deqn{\pi(\sigma, \xi) = (1/ \sigma), for \sigma > 0.}
 #'
-#'     For \code{model = "ev"}: a flat prior for
+#'     For \code{model = "gev"}: a flat prior for
 #'     \eqn{\xi} (and for \eqn{\mu} and \eqn{log \sigma}):
 #'     \deqn{\pi(\mu, \sigma, \xi) = (1/ \sigma), for \sigma > 0.}
 #'   }
 #'   \item{\code{"flatflat"}.
 #'
-#'     For \code{model = "gp"}: flat priors for \eqn{\sigma}
-#'     and \eqn{\xi}:
+#'     For \code{model = "gp"}: flat priors for
+#'     \eqn{\sigma} and \eqn{\xi}:
 #'     \deqn{\pi(\sigma, \xi) = 1, for \sigma > 0.}
 #'
 #'     For \code{model = "gev"}: flat priors for \eqn{\mu}, \eqn{\sigma}
@@ -120,8 +124,8 @@
 #'
 #'     Therefore, the posterior is proportional to the likelihood.
 #'   }
-#'   \item{\code{"jeffreys"}.  For \code{model = "gp"} only: the
-#'     Jeffreys prior (Castellanos and Cabras, 2007):
+#'   \item{\code{"jeffreys"}.  For \code{model = "gp"} only: the Jeffreys
+#'     prior (Castellanos and Cabras, 2007):
 #'     \deqn{\pi(\sigma, \xi) = 1/ [\sigma (1+\xi) \sqrt(1+2\xi)],
 #'       for \sigma > 0, \xi > -1 / 2.}
 #'
@@ -129,8 +133,8 @@
 #'     for any sample size.  See Northrop and Attalides (2016) for details.
 #'   }
 #'   \item{\code{"beta"}.
-#'     For \code{model = "gp"}: a beta-type(p, q) prior is used for xi
-#'     on the interval (\code{min_xi}, \code{max_xi}):
+#'     For \code{model = "gp"}: a beta-type(p, q)
+#'     prior is used for xi on the interval (\code{min_xi}, \code{max_xi}):
 #'     \deqn{\pi(\sigma, \xi) = (1/\sigma) (\xi - min_xi) ^ (p-1)
 #'           (max_xi - \xi) ^ (q-1), for min_xi < \xi < max_xi.}
 #'
@@ -144,9 +148,10 @@
 #'     prior for \eqn{\xi} proposed in Martins and Stedinger (2000, 2001).
 #'   }
 #' }
-#' @return A list.  The first component is the input prior, i.e. either the
-#'   name of the prior or a user-supplied function.  The remaining components
-#'   contain the numeric values of any hyperparameters in the prior.
+#' @return A list with class \code{"evprior"}.  The first component is the
+#'   input prior, i.e. either the name of the prior or a user-supplied
+#'   function.  The remaining components contain the numeric values of any
+#'   hyperparameters in the prior.
 #' @seealso \code{\link{rpost}} for sampling from an extreme value posterior
 #'   distribution.
 #' @seealso \code{\link[evdbayes]{prior.prob}},
@@ -199,8 +204,12 @@
 #' @export
 set_prior <- function(prior = c("norm", "loglognorm", "mdi", "flat",
                                 "flatflat", "jeffreys", "beta"),
-                      model = c("gp", "gev", "pp", "os"),
-                      ...) {
+                      model = c("gev", "gp", "pp", "os"), ...) {
+  if (length(model) > 1) {
+    warning("model not supplied: model == \"gev\" has been used.",
+            immediate. = TRUE)
+  }
+  model <- match.arg(model)
   # If prior is a function then just return it in the required format,
   # together with any additional arguments from ....
   if (is.function(prior)) {
@@ -217,7 +226,6 @@ set_prior <- function(prior = c("norm", "loglognorm", "mdi", "flat",
     return(structure(temp, class = "evprior", model = model))
   }
   # Otherwise, call the appropriate function to set the prior with name prior.
-  model <- match.arg(model)
   prior <- match.arg(prior)
   temp <- switch(model, gp = gp_prior(prior, ...), gev = gev_prior(prior, ...),
           pp = gev_prior(prior, ...), os = gev_prior(prior, ...))
@@ -231,8 +239,8 @@ gp_prior <- function(prior = c("norm", "mdi", "flat", "flatflat", "jeffreys",
   prior <- match.arg(prior)
   temp <- list(prior = paste("gp_", prior, sep=""), ...)
   # Check for unused hyperparameter names and drop them
-  hpar_vec <- switch(prior, norm = c("mean", "cov"), mdi = "a_mdi",
-                     flat = NULL, jeffreys = NULL, beta = "ab")
+  hpar_vec <- switch(prior, norm = c("mean", "cov"), mdi = "a",
+                     flat = NULL, jeffreys = NULL, beta = "pq")
   hpar_vec <- c(hpar_vec, "min_xi", "max_xi")
   temp <- hpar_drop(temp, hpar_vec)
   # Check for problems with min_xi and/or max_xi
@@ -269,10 +277,10 @@ gp_prior <- function(prior = c("norm", "mdi", "flat", "flatflat", "jeffreys",
     temp$cov <- NULL
     temp <- c(temp, list(icov=icov))
   }
-  if (prior == "mdi" & !is.null(temp$a_mdi)) {
-    a_mdi <- temp$a_mdi
-    if (length(a_mdi) != 1 | !is.numeric(a_mdi) | a_mdi <= 0)
-        stop("a_mdi must be a positive numeric vector of length 1")
+  if (prior == "mdi" & !is.null(temp$a)) {
+    a <- temp$a
+    if (length(a) != 1 | !is.numeric(a) | a <= 0)
+        stop("a must be a positive numeric vector of length 1")
   }
   if (prior == "beta" & !is.null(temp$pq)) {
     pq <- temp$pq
@@ -343,8 +351,8 @@ gev_prior <- function(prior=c("norm", "loglognorm", "mdi", "flat", "flatflat",
   prior <- match.arg(prior)
   temp <- list(prior = paste("gev_", prior, sep=""), ...)
   # Check for unused hyperparameter names and drop them
-  hpar_vec <- switch(prior, norm = c("mean", "cov"), mdi = "a_mdi",
-                      flat = NULL, beta = "ab")
+  hpar_vec <- switch(prior, norm = c("mean", "cov"), mdi = "a",
+                      flat = NULL, beta = "pq")
   hpar_vec <- c(hpar_vec, "min_xi", "max_xi")
   temp <- hpar_drop(temp, hpar_vec)
   # Check for problems with min_xi and/or max_xi
@@ -376,10 +384,10 @@ gev_prior <- function(prior=c("norm", "loglognorm", "mdi", "flat", "flatflat",
     temp$cov <- NULL
     temp <- c(temp, list(icov=icov))
   }
-  if (prior == "mdi" & !is.null(temp$a_mdi)) {
-    a_mdi <- temp$a_mdi
-    if (length(a_mdi) != 1 | !is.numeric(a_mdi) | a_mdi <= 0)
-        stop("a_mdi must be a positive numeric vector of length 1")
+  if (prior == "mdi" & !is.null(temp$a)) {
+    a <- temp$a
+    if (length(a) != 1 | !is.numeric(a) | a <= 0)
+        stop("a must be a positive numeric vector of length 1")
   }
   if (prior == "beta" & !is.null(temp$pq)) {
     pq <- temp$pq
@@ -421,12 +429,12 @@ gev_loglognorm <- function(pars, mean, icov, min_xi = -Inf, max_xi = Inf,
   return(-ld / 2 - pars[2] - pars[3])
 }
 
-gev_mdi <- function(pars, a_mdi=0.5772156649015323, min_xi=-1, max_xi=Inf,
+gev_mdi <- function(pars, a=0.5772156649015323, min_xi=-1, max_xi=Inf,
                     trendsd = 0) {
   if (pars[2] <= 0 | pars[3] < min_xi | pars[3] > max_xi) {
     return(-Inf)
   }
-  return(-log(pars[1]) - a_mdi * pars[3])
+  return(-log(pars[1]) - a * pars[3])
 }
 
 gev_flat <- function(pars, min_xi = -Inf, max_xi = Inf, trendsd = 0) {
@@ -472,9 +480,76 @@ hpar_drop <- function(x_list, hpar_vec) {
     warning("The following user-supplied arguments are unused and have been
             dropped:", immediate. = TRUE)
   }
-  cat(names(x_list[to_drop]), "\n")
   if (length(to_drop) > 0) {
+    cat(names(x_list[to_drop]), "\n")
     x_list <- x_list[-to_drop]
   }
   return(x_list)
 }
+
+# ============================== set_bin_prior ================================
+
+#' Construction of a prior distribution for a binomial probability \eqn{p}
+#'
+#' Constructs a prior distribution for use as the argument \code{bin_prior} in
+#' \code{\link{rpost}} or in \code{\link{binpost}}.  The user can either
+#' specify their own prior function and arguments for hyperparameters or choose
+#' from a list of in-built priors.
+#'
+#' @param prior A character string giving the name of the prior for \eqn{p}.
+#'   See \strong{Details} for a list of the priors available.
+#' @param ... Further arguments to be passed to the user-supplied or
+#'   in-built prior function.  For details of the latter see \strong{Details}.
+#' @details
+#'   \strong{Binomial priors.} The names of the binomial priors set using
+#'   \code{bin_prior} are:
+#' \itemize{
+#'   \item {\code{"jeffreys"}: the \emph{Jeffreys} beta(1/2, 1/2) prior.}
+#'   \item {\code{"laplace"}: the \emph{Bayes-Laplace} beta(1, 1) prior.}
+#'   \item{\code{"haldane"}: the \emph{Haldane} beta(0, 0) prior.}
+#'   \item{\code{"beta"}: a beta(\eqn{\alpha, \beta}) prior.  The argument
+#'     \code{ab} is a vector containing \code{c}(\eqn{\alpha, \beta}).}
+#'     The default is \code{ab = c(1, 1)}.
+#'   \item{\code{"mdi"}: the MDI prior
+#'     \eqn{\pi(p) = 1.6186 p^p (1-p)^(1-p),     0 < p < 1.}
+#'   }
+#' }
+#' Apart from the MDI prior these are all beta distributions.
+#' @return A list of class \code{"binprior"}.  The first component is the
+#'   name of the input prior.  Apart from the MDI prior this will be "beta",
+#'   in which case the other component of the list is a vector of length two
+#'   giving the corresponding values of the beta parameters.
+#' @seealso \code{\link{binpost}} for sampling from a binomial posterior
+#'   distribution.
+#' @examples
+#' bp <- set_bin_prior(prior = "jeffreys")
+#' @export
+set_bin_prior <- function(prior = c("jeffreys", "laplace", "haldane", "beta",
+                                    "mdi"), ...) {
+  prior <- match.arg(prior)
+  temp <- list(prior = paste("bin_", prior, sep=""), ...)
+  # Check for unused hyperparameter names and drop them
+  if (prior == "beta") {
+    hpar_vec <- "ab"
+  } else {
+    hpar_vec <- NULL
+  }
+  temp <- hpar_drop(temp, hpar_vec)
+  # Check admissibility of hyperparameters
+  if (prior == "beta" & !is.null(temp$ab)) {
+    ab <- temp$ab
+    if (length(ab) != 2 | !is.numeric(ab) | any(ab <= 0) )
+      stop("ab must be a non-negative numeric vector of length 2")
+  }
+  if (prior == "beta" & is.null(temp$ab)) {
+    temp$ab <- c(1, 1)
+  }
+  # Jeffreys, Laplace and Haldane prior are all beta distributions.
+  temp$ab <- switch(prior, jeffreys = c(1/2, 1/2), laplace = c(1, 1),
+               haldane = c(0, 0), beta = temp$ab)
+  if (prior %in% c("jeffreys", "laplace", "haldane")) {
+    temp$prior <- paste("bin_", "beta", sep = "")
+  }
+  return(structure(temp, class = "binprior"))
+}
+
